package com.example.e13roomshoppinglist


import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.horizontalScroll
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.Divider
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.RadioButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp


@Composable
fun ShoppingListScreen(
    state: ShoppingListState,
    onEvent: (ShoppingListEvent) -> Unit,
    onShowToast: (String) -> Unit?
) {
    Scaffold(
        floatingActionButton = {
            FloatingActionButton(onClick = {
                onEvent(ShoppingListEvent.OpenAddingNewShoppingListItemDialog)
            }) {
                Icon(imageVector = Icons.Default.Add, contentDescription = "Add new shopping list item")
            }
        },
        modifier = Modifier.padding(16.dp)
    ) { padding ->
        if (state.isAddingNewShoppingListItem || state.isEditingShoppingListItem) {
            AddShoppingListItemDialog(state = state, onEvent = onEvent, onShowToast = onShowToast)
        }
        Column {


            Row(
                modifier = Modifier
                    .fillMaxWidth(),
                verticalAlignment = Alignment.CenterVertically
            ) {
                Row(Modifier.padding(end = 8.dp)) {
                    Text(text = "Sort by:", fontSize = 20.sp, fontWeight = FontWeight.Bold)
                }
                Row(Modifier.horizontalScroll(rememberScrollState())) {
                    SortType.entries.forEach { sortType ->
                        Row(
                            modifier = Modifier
                                .clickable { onEvent(ShoppingListEvent.SetSortType(sortType)) },
                            verticalAlignment = Alignment.CenterVertically
                        ) {
                            RadioButton(
                                selected = state.sortType == sortType,
                                onClick = { onEvent(ShoppingListEvent.SetSortType(sortType)) })
                            Text(text = sortType.name, fontSize = 16.sp)
                        }
                    }
                }
            }
            Divider(thickness = 2.dp, color = Color.LightGray, modifier = Modifier.padding(bottom = 8.dp))

            LazyColumn(
                contentPadding = padding,
                modifier = Modifier.fillMaxSize(),
                verticalArrangement = Arrangement.spacedBy(16.dp)
            ) {
                items(state.shoppingListItems) { shoppingItem ->
                    Row(modifier = Modifier.fillMaxWidth()) {
                        Column(modifier = Modifier.weight(1f)) {
                            Text(
                                text = "${shoppingItem.name} - 🧮 ${shoppingItem.count}",
                                fontSize = 18.sp
                            )
                            Text(text = "💶 ${shoppingItem.price}", fontSize = 16.sp)
                        }

                        IconButton(onClick = {
                            onEvent(ShoppingListEvent.DeleteShoppingListItem(shoppingItem))
                                .also {
                                    onShowToast("Deleted shopping list item ${shoppingItem.name}")
                                }
                        }) {
                            Icon(
                                imageVector = Icons.Default.Delete,
                                contentDescription = "Delete Shopping List Item"
                            )
                        }

                        IconButton(onClick = {
                            onEvent(ShoppingListEvent.OpenEditingShoppingListItem(shoppingItem))
                        }) {
                            Icon(
                                imageVector = Icons.Default.Edit,
                                contentDescription = "Edit Shopping List Item"
                            )
                        }
                    }
                }
                item {
                    Row(
                        modifier = Modifier
                            .width(280.dp)
                            .height(70.dp),
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Text(text = "Empty item to fill up room so that the last item does not go under FAB.", fontSize = 12.sp, color = Color.Gray)
                    }
                }
            }
        }
    }
}

@Composable
fun AddShoppingListItemDialog(
    state: ShoppingListState,
    onEvent: (ShoppingListEvent) -> Unit,
    onShowToast: (String) -> Unit?,
    modifier: Modifier = Modifier,
) {
    AlertDialog(
        modifier = modifier,
        onDismissRequest = {
            if (state.isAddingNewShoppingListItem) {
                onEvent(ShoppingListEvent.CloseAddingNewShoppingListItemDialog)
            } else if (state.isEditingShoppingListItem) {
                onEvent(ShoppingListEvent.CloseEditingShoppingListItem)
            }
        },
        title = { Text(text = "Add new shopping list item") },
        text = {
            Column(verticalArrangement = Arrangement.spacedBy(8.dp)) {
                val shoppingListItem = state.newShoppingListItem

                TextField(
                    value = shoppingListItem.name,
                    onValueChange = { onEvent(ShoppingListEvent.SetName(it)) },
                    label = { Text(text = "Item Name") },
                    placeholder = { Text(text = "Item Name") },
                )
                TextField(
                    value = if(shoppingListItem.count>0) shoppingListItem.count.toString() else "",
                    label = { Text(text = "Count") },
                    placeholder = { Text(text = "Count") },
                    onValueChange = {
                        var count = it.toIntOrNull()
                        if (count===null) {
                            onShowToast("Invalid value for Count!")
                            count = 0
                        }
                        onEvent(ShoppingListEvent.SetCount(count = count))
                    },
                    keyboardOptions = KeyboardOptions().copy(
                        autoCorrect = false,
                        keyboardType = KeyboardType.Number
                    ),
                    leadingIcon = {
                        Text(text = "🧮", fontSize = 24.sp)
                    }
                )
                TextField(
                    value = if(shoppingListItem.price>0) shoppingListItem.price.toString() else "",
                    label = { Text(text = "Price") },
                    placeholder = { Text(text = "Price") },
                    onValueChange = {
                        var price = it.toFloatOrNull()
                        if (price===null) {
                            price = 0f
                        }
                        onEvent(ShoppingListEvent.SetPrice(price = price))
                    },
                    keyboardOptions = KeyboardOptions().copy(
                        autoCorrect = false,
                        keyboardType = KeyboardType.Decimal
                    ),
                    leadingIcon = {
                        Text(text = "💶", fontSize = 24.sp)
                    }
                )
            }
        },
        confirmButton = {
            Button(onClick = {
                if (state.newShoppingListItem.name.length<3){
                    onShowToast("Item name should not be too short!")
                    return@Button
                }
                if (state.isAddingNewShoppingListItem) {
                    onEvent(ShoppingListEvent.InsertNewShoppingListItemToDb).also { onShowToast("Added new item to shopping list!") }
                } else if (state.isEditingShoppingListItem) {
                    onEvent(ShoppingListEvent.UpdateCurrentlyEditingShoppingListItem).also { onShowToast("Updated shopping list item!") }
                }
            }) {
                Text(text = if (state.isEditingShoppingListItem) "Update" else "Insert")
            }
        },
        dismissButton = {
            Button(onClick = {
                if (state.isAddingNewShoppingListItem) {
                    onEvent(ShoppingListEvent.CloseAddingNewShoppingListItemDialog)
                } else if (state.isEditingShoppingListItem) {
                    onEvent(ShoppingListEvent.CloseEditingShoppingListItem)
                }
            }) {
                Text(text = "Close")
            }
        }
    )
}